#ifndef __DAQERRORMSG
#define __DAQERRORMSG 
#include <cstdlib>
#include <ctime>
#include <unistd.h>
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>

/** \file daqerrormesg.h
    \brief  Error message handling and debug information handling 

    The DT_DEBUG_XXX  is a macro that is something like
    cout << function info << line info  << what you type between the brackets.

    During compilation the environment variable LTDEBUG has to be set.
    export LTDEBUG=1
    (so put it in your .bashrc in for your evelopment accounts).

    If this is done the macro becomes active if the environment variable DAQPRINTINGLEVEL is set. 
    export DAQPRINTINGLEVEL=0xnumber
    The DAQPRINTINGLEVEL is set by default to 1. If you set it to 0 the DT_WARN messages will also be suppressed. 

    The number is an OR from the debug info you want to see
    For record analyzing 
    DT_DEBUG_RANA=0x8000,
    So if you do DT_DEBUG_RANA( "this is the the message");
    set
    export DAQPRINTINGLEVEL=0x8000
    To see also the data base debug information (DT_DEBUG_DB=0x800 )
    export DAQPRINTINGLEVEL=0x8800

    To on by default ( so if without the need to set
    export DAQPRINTINGLEVEL=0x8000  ) the line 
    DaqToolsErrorMessage dteinit(0x8000);
    can be added in the main program. This will overwrite the DAQPRINTINGLEVEL environment !!!. 

    Advantage of DT_DEBUG is
    - switch it on if needed
    - select the messages what is needed 
    - not needed to remove them or comment out the debug information to suppress.
    - The messages are not in the release binaries ( macro is set to empty so no code generated if LTDEBUG is not set)
    - standard way of outputting messages .
    Disadvantage: a little bit less flexible as cout

    The exception is DT_WARN, DT_ERROR , DT_EXIT
    These messages are always printed. These messagages also go to the log file.

    The principle from the DT_DEBUG comes from the Tridas project.

    .
              
     \author W. Beaumot 
     \version 2.0 
     \date 01/2004
 
*/



// globals
enum DaqToolsMessageLevel  { DT_ERROR_LEVEL=0  ,
			     DT_WARNING_LEVEL=1,
			     DT_NOTE_LEVEL=2, 
			     DT_DEBUG_LEVEL=0x4,
			     DT_DEBUG_LAST=0x4,
			     DT_DEBUG_SP=0x8 , //set parser 
			     DT_DEBUG_LP=0x10,      // lt parser
			     DT_DEBUG_LTC=0x20,     // lt tools
			     DT_DEBUG_LTAPPL=0x40,  //common application 
			     DT_DEBUG_SRV= 0x00080, //related to servers
			     DT_DEBUG_PROC=0x00100, //related to loops 
			     DT_DEBUG_REC= 0x00200, //related to records
			     DT_DEBUG_MUX= 0x00400, //related to f-mux
			     DT_DEBUG_DB=  0x00800, //related to data base
			     DT_DEBUG_DBW= 0x01000, //related to data base write
			     DT_DEBUG_GUI= 0x02000, //related to the GUI
			     DT_DEBUG_THR= 0x04000, // relate to threads
			     DT_DEBUG_RANA=0x08000, // related to analyzaer
			     DT_DEBUG_ND3= 0x10000, // for new develop , remove
			     DT_DEBUG_ALL=0x1FFFF		     	     
			     
                      };
  


// define to expand debug statements from code
#ifdef LTDEBUG1 
#define DT_DEBUG_ENABLE
#endif 
/***********************************************************************
 * printing
 ***********************************************************************/
class DaqToolsErrorMessage{ 
  private : 
    static int printing_level;
  void OpenLogFile();
  char envvar[40];
  public :
    DaqToolsErrorMessage(){
    if(printing_level == -1){ // first call   
      if ( getenv( "DAQPRINTINGLEVEL")){
	strcpy(envvar,   getenv("DAQPRINTINGLEVEL") );
        printing_level= (int)strtol( envvar,0,0);
      }
      else { printing_level = 1; // default value}
      }
      printing_level|= 0; // set always on values
    }
    if(!logfile) { OpenLogFile();}
    // if printing level != -1 it will be already initialized.
  }
  std::ostringstream ss;
  DaqToolsErrorMessage(int i ){ 
    printing_level=i;
    if(!logfile) { OpenLogFile();}
  }
  ~DaqToolsErrorMessage(){logfile->flush();}
  void setPrintingLevel(int i){ printing_level=i;}
  int getPrintingLevel(){ return printing_level;}
  static std::ofstream* logfile;
  std::string  getTime() {
    time_t now= std::time(0);
    std::string s=std::asctime(std::localtime(&now));
    s=s.substr(4, 16);
    s=s+": ";
    return  s;
  }
};

#define DT_DEBUG_TEMPL(args,value)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & value )  \
   {  std::cout << __FILE__<<":"<< __FUNCTION__<<":"<< __LINE__<<": ";       \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }	}							       



#ifdef  DT_DEBUG_ENABLE
#define DT_DEBUG(args)						\
do { DaqToolsErrorMessage dte;		                        \
   if (dte.getPrintingLevel() >= DT_DEBUG_LEVEL  )				\
   {	std::cout <<	 __FILE__<<":"<< __FUNCTION__  <<" module "<< 	__FILE__ <<" at line "<< __LINE__;\
       std::cout <<" "<<  args;						        \
       std::cout << std::endl;						        \
	     }								        \
} while (0);

#define DT_DEBUG_REC(args)    DT_DEBUG_TEMPL(args,DT_DEBUG_REC)
#define DT_DEBUG_LTAPPL(args) DT_DEBUG_TEMPL(args,DT_DEBUG_LTAPPL)
#define DT_DEBUG_PROC(args)   DT_DEBUG_TEMPL(args,DT_DEBUG_PROC )
#define DT_DEBUG_SP(args)     DT_DEBUG_TEMPL(args,DT_DEBUG_SP)
#define DT_DEBUG_LP(args)     DT_DEBUG_TEMPL(args,DT_DEBUG_LP )
#define DT_DEBUG_LTC(args)    DT_DEBUG_TEMPL(args, DT_DEBUG_LTC)
#define DT_DEBUG_SRV(args)    DT_DEBUG_TEMPL(args, DT_DEBUG_SRV)
#define DT_DEBUG_ND3(args)    DT_DEBUG_TEMPL(args, DT_DEBUG_ND3) 
#define DT_DEBUG_MUX(args)    DT_DEBUG_TEMPL(args, DT_DEBUG_MUX )
#define DT_DEBUG_DB(args)     DT_DEBUG_TEMPL(args, DT_DEBUG_DB ) 
#define DT_DEBUG_DBW(args)    DT_DEBUG_TEMPL(args, & DT_DEBUG_DBW )
#define DT_DEBUG_GUI(args)    DT_DEBUG_TEMPL(args,DT_DEBUG_GUI )
#define DT_DEBUG_THR(args)    DT_DEBUG_TEMPL(args,DT_DEBUG_THR ) 
#define DT_DEBUG_RANA(args)   DT_DEBUG_TEMPL(args,DT_DEBUG_RANA )


#else
#define DT_DEBUG(args)	
#define DT_DEBUG_SP(args) 
#define DT_DEBUG_LP(args)
#define DT_DEBUG_LTC(args) 
#define DT_DEBUG_LTAPPL(args) 
#define DT_DEBUG_SRV(args) 
#define DT_DEBUG_PROC(args) 
#define DT_DEBUG_ND3(args) 
#define DT_DEBUG_MUX(args)
#define DT_DEBUG_DB(args)
#define DT_DEBUG_GUI(args)
#define DT_DEBUG_THR(args)
#define DT_DEBUG_RANA(args)
#endif
#define DAQEXIT(args) {  \
  DaqToolsErrorMessage dte; dte.ss<<std::endl; \
  dte.ss <<dte.getTime(); \
  dte.ss << args ; \
  dte.ss << std::endl <<  "****exit at "  << __FILE__ << " in function " ;\
  dte.ss <<  __FILE__<<":"<< __FUNCTION__ << " at line " << __LINE__    << std::endl;\
  std::cout <<dte.ss.str(); if(dte.logfile) {*dte.logfile <<dte.ss.str();}  \
  std::exit(1);  \
} ;
 
#define DT_LOG(args)	\
do {	\
   DaqToolsErrorMessage dte; dte.ss <<dte.getTime();\
   dte.ss <<  "***Log in function " ;\
   dte.ss <<  __FILE__<<":"<< __FUNCTION__ <<"*** "; dte.ss << args ;\
   if(dte.logfile) {*dte.logfile <<dte.ss.str();}  \
} while (0);


#define DT_PANIC(args)	\					\
do {	\
   DaqToolsErrorMessage dte; dte.ss <<dte.getTime(); dte.ss << args ;\
   dte.ss << std::endl <<  "****Panic exit at "  << __FILE__ << " in function " ;\
   dte.ss <<  __PRETTY_FUNCTION__ << " at line " << __LINE__    << std::endl;\
   std::cout <<dte.ss.str(); \
   if(dte.logfile) {*dte.logfile <<dte.ss.str();}  \
 	  std::exit(1);								\
} while (0);

#define DT_NOTE(args)  { \
DaqToolsErrorMessage dte;                      \
if (dte.getPrintingLevel()>= DT_NOTE_LEVEL) {	   \
    dte.ss <<dte.getTime()<< __FUNCTION__ ;\
    dte.ss << args; dte.ss <<std::endl;		 \
     std::cout <<dte.ss.str(); if(dte.logfile) {*dte.logfile <<dte.ss.str();}  \
    }	}								


#define DT_WARN(args)  do {	\
  DaqToolsErrorMessage dte;		        \
  if (dte.getPrintingLevel() >=DT_WARNING_LEVEL)			\
    {  dte.ss <<dte.getTime();\
      dte.ss << "*** Warning in " << __FUNCTION__ << " module " \
            << __FILE__<< " at "<< __LINE__ <<"\t";\
       dte.ss << args;				     \
       dte.ss << std::endl;				\
   std::cout <<dte.ss.str(); if(dte.logfile) {*dte.logfile <<dte.ss.str();}  \
  }							\
} while (0);

#define DT_ERROR(args)						\
do {	DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()	>= DT_ERROR_LEVEL)		        \
	{	 dte.ss <<dte.getTime(); \
           dte.ss <<"*** Error in"  << __FILE__<<":"<< __FUNCTION__ <<" module " << __FILE__ << " at line " << __LINE__;\
     		dte.ss << args;						\
		dte.ss << std::endl ;   		\
			}				         \
  std::cout <<dte.ss.str(); if(dte.logfile) {*dte.logfile <<dte.ss.str();}  \
} while (0);

#endif

